"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const disposable_1 = require("@reactive-js/disposable");
const option_1 = require("@reactive-js/option");
const scheduler_1 = require("@reactive-js/scheduler");
const performance = window.performance;
const Date = window.Date;
const setTimeout = window.setTimeout;
const clearTimeout = window.clearTimeout;
const now = typeof performance === "object" && typeof performance.now === "function"
    ? () => performance.now()
    : () => Date.now();
const yieldInterval = 5;
const maxYieldInterval = 300;
const callCallbackAndDispose = (callback, disposable) => {
    startTime = now();
    callback(shouldYield);
    disposable.dispose();
};
const scheduleImmediate = (callback) => {
    const disposable = disposable_1.createDisposable();
    channel.port1.onmessage = () => {
        if (!disposable.isDisposed) {
            startTime = now();
            callback(shouldYield);
            disposable.dispose();
        }
    };
    channel.port2.postMessage(null);
    return disposable;
};
const scheduleDelayed = (callback, delay) => {
    const disposable = disposable_1.createDisposable(() => clearTimeout(timeout));
    const timeout = setTimeout(callCallbackAndDispose, delay, callback, disposable);
    return disposable;
};
const shouldYield = option_1.isSome(navigator) &&
    option_1.isSome(navigator.scheduling) &&
    option_1.isSome(navigator.scheduling.isInputPending)
    ? () => {
        const currentTime = now();
        const deadline = startTime + yieldInterval;
        const maxDeadline = startTime + maxYieldInterval;
        const inputPending = navigator.scheduling.isInputPending();
        return ((currentTime >= deadline && inputPending) ||
            currentTime >= maxDeadline);
    }
    : () => now() >= startTime + yieldInterval;
const channel = new MessageChannel();
const schedulerImpl = {
    inContinuation: false,
    get now() {
        return now();
    },
    schedule: scheduler_1.schedule,
    scheduleCallback(callback, delay) {
        return delay >= 4
            ? scheduleDelayed(callback, delay)
            : scheduleImmediate(callback);
    },
};
let startTime = now();
exports.scheduler = schedulerImpl;
//# sourceMappingURL=scheduler.js.map