import { createAsyncEnumerable, } from "@reactive-js/async-enumerable";
import { compute, merge, onNotify, throttle, } from "@reactive-js/observable";
import { none } from "@reactive-js/option";
import { pipe } from "@reactive-js/pipe";
import { fromEvent } from "./event";
const locationEquals = (a, b) => a === b ||
    (a.pathname === b.pathname && a.search === b.search && a.hash === b.hash);
const getCurrentLocation = (_) => {
    const pathname = window.location.pathname;
    const search = window.location.search;
    const hash = window.location.hash;
    return { pathname, search, hash };
};
const pushHistoryState = (newLocation) => {
    const currentLocation = getCurrentLocation();
    if (!locationEquals(currentLocation, newLocation)) {
        const { pathname, search, hash } = newLocation;
        let uriString = pathname;
        uriString = search.length > 0 ? `${uriString}${search}` : uriString;
        uriString = hash.length > 0 ? `${uriString}${hash}` : uriString;
        window.history.pushState(none, "", uriString);
    }
};
const historyOperator = (obs) => merge(compute(getCurrentLocation), pipe(obs, throttle(15), onNotify(pushHistoryState)), fromEvent(window, "popstate", getCurrentLocation));
const _history = createAsyncEnumerable(historyOperator);
export const history = _history;
//# sourceMappingURL=history.js.map